package com.gymtracker.handler;

import com.gymtracker.util.HibernateUtil;
import com.gymtracker.util.Util;
import com.gymtracker.model.Logging;
import io.github.cdimascio.dotenv.Dotenv;
import org.hibernate.Session;
import org.hibernate.SessionFactory;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPFault;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import java.sql.Timestamp;
import java.util.*;
// import javax.xml.soap.SOAPMessage;
// import java.util.stream.Collectors;

public class LogAuthHandler implements SOAPHandler<SOAPMessageContext> {
    @Override
    public Set<QName> getHeaders() {
        return null;
    }

    @Override
    public void close(MessageContext mc) {
    }

    public boolean handleFault(SOAPMessageContext smc) {
        return true;
    }

    public boolean handleMessage(SOAPMessageContext smc) {
        try {
            Boolean outbound = (Boolean) smc.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);
            boolean authorized = authorize(smc);
            if (!outbound) {
                log(smc, authorized);
            }
            if(!authorized) {
                // Set 400 error code 
                smc.put(MessageContext.HTTP_RESPONSE_CODE, 400);

                SOAPBody body = smc.getMessage().getSOAPBody();
                body.removeContents();
                SOAPFault soapFault = body.addFault();
                soapFault.setFaultCode("Access");
                soapFault.setFaultString("Unauthorized access");

                return false;
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    private boolean authorize(SOAPMessageContext smc) {
        try {
            @SuppressWarnings("unchecked")
            Map<String, List<?>> httpHeaders = (Map<String, List<?>>) smc.get(MessageContext.HTTP_REQUEST_HEADERS);
            Dotenv dotenv = Dotenv.configure().directory("/").load();

            if (httpHeaders.containsKey("Authorization")) {
                String token = (String) httpHeaders.get("Authorization").get(0);

                String phpKey = dotenv.get("PHP_KEY");
                String nodeKey = dotenv.get("NODE_KEY");

                if(token.equals(phpKey) || token.equals(nodeKey)) {
                    return true;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    private void log(SOAPMessageContext smc, boolean authorized) {
        try {
            StringBuilder description = new StringBuilder();
            String formattedBodyContent = Util.formatContentBody(smc);

            if(authorized) {
                @SuppressWarnings("unchecked")
                Map<String, List<?>> httpHeaders = (Map<String, List<?>>) smc.get(MessageContext.HTTP_REQUEST_HEADERS);
                Dotenv dotenv = Dotenv.configure().directory("/").load();
                String token = (String) httpHeaders.get("Authorization").get(0);
                String phpKey = dotenv.get("PHP_KEY");

                description.append("Authorized access; ");
                String caller = token.equals(phpKey) ? "PHP" : "Node";
                description.append(caller).append(" call to ");

                QName operation = (QName) smc.get(MessageContext.WSDL_OPERATION);
                description.append(operation.getLocalPart());

            } else {
                description.append("Unauthorized access; Tried to call ");
                QName operation = (QName) smc.get(MessageContext.WSDL_OPERATION);
                description.append(operation.getLocalPart());

            }
            description.append(" with arguments:\n");
            description.append(formattedBodyContent);

            Logging logging = new Logging();
            logging.setRequestDescription(description.toString());
            logging.setEndpoint(Util.getURL(smc));
            logging.setIPAddress(Util.getIP(smc));
            logging.setTimestamp(Util.toUTCSeven(new Timestamp(System.currentTimeMillis())));

            SessionFactory sessionFactory = HibernateUtil.getSessionFactory();
            Session session = sessionFactory.getCurrentSession();

            session.beginTransaction();
            session.save(logging);
            session.getTransaction().commit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
