package com.gymtracker.service;

import com.gymtracker.model.Application;

import com.gymtracker.util.HibernateUtil;
import org.hibernate.Session;
import org.hibernate.SessionFactory;

import javax.annotation.Resource;
import javax.jws.HandlerChain;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.xml.ws.WebServiceContext;

import java.util.List;
// import javax.mail.MessageContext;
// import javax.xml.soap.SOAPException;
// import javax.xml.soap.SOAPFactory;
// import javax.xml.soap.SOAPFault;
// import javax.xml.ws.soap.SOAPFaultException;
// import javax.mail.MessageContext;
// import javax.xml.bind.JAXB;
// import javax.xml.ws.handler.soap.SOAPMessageContext;
// import java.io.StringWriter;
// import javax.xml.soap.SOAPMessage;
// import javax.xml.ws.handler.MessageContext;
// import javax.xml.ws.handler.soap.SOAPMessageContext;
// import java.util.Map;

@WebService
@HandlerChain(file = "log_and_auth.xml")
public class ApplicationService {
    @Resource
    private WebServiceContext wsContext;

    private static Session getSession() {
        SessionFactory sessionFactory = HibernateUtil.getSessionFactory();
        return sessionFactory.getCurrentSession();
    }

    private static void beginTransaction(Session session) {
        if (session != null && !session.getTransaction().isActive()) {
            session.beginTransaction();
        }
    }

    private static void commitTransaction(Session session) {
        if (session != null && session.getTransaction().isActive()) {
            session.getTransaction().commit();
        }
    }

    private static void rollbackTransaction(Session session) {
        if (session != null && session.getTransaction().isActive()) {
            session.getTransaction().rollback();
        }
    }

    private static Application getApplication(Session session, String username, int gym_id) {
        return (Application) session.createQuery("FROM Application WHERE username = :username AND gym_id = :gym_id")
                .setParameter("username", username)
                .setParameter("gym_id", gym_id)
                .uniqueResult();
    }

    // private SOAPFaultException createSoapFaultException(String errorMessage) {
    //     try {
    //         SOAPFactory soapFactory = SOAPFactory.newInstance();
    //         SOAPFault soapFault = soapFactory.createFault();
    //         soapFault.setFaultString(errorMessage);
    //         return new SOAPFaultException(soapFault);
    //     } catch (SOAPException e) {
    //         throw new RuntimeException("Error creating SOAPFaultException", e);
    //     }
    // }

    @WebMethod
    public String apply (
            @WebParam(name = "username") String username,
            @WebParam(name = "gym_id") int gym_id,
            @WebParam(name = "gym_name") String gym_name,
            @WebParam(name = "email") String email,
            @WebParam(name = "trainer_name") String trainer_name,
            @WebParam(name = "trainer_description") String trainer_description,
            @WebParam(name = "application_description") String application_description
    ) throws Exception {
        try {
            Application application = new Application(username, gym_id, gym_name, email, 0, trainer_name);
            application.setTrainerDescription(trainer_description);
            application.setApplicationDescription(application_description);
            System.out.println(application.toString());
            
            Session session = ApplicationService.getSession();

            ApplicationService.beginTransaction(session);
            session.save(application);
            ApplicationService.commitTransaction(session);

            return "Application successful with username: " + username + " and gym_id: " + gym_id;
        } catch (Exception e) {
            ApplicationService.rollbackTransaction(getSession());
            e.printStackTrace();
            // throw createSoapFaultException("Failed: Application failed with error: " + e.getMessage());
            // return "Failed: Application failed with error: " + e.getMessage();
            throw new Exception("Failed: Application failed with error: " + e.getMessage());
        }
    }

    @WebMethod
    public String accept(
        @WebParam(name = "username") String username,
        @WebParam(name = "gym_id") int gym_id
    ) throws Exception {
        try {
            // set the application acceptance to 1 and make the other applications for the same gym rejected with 2
            Session session = ApplicationService.getSession();
            ApplicationService.beginTransaction(session);

            // Update the acceptance status of the application based on the gym_id (1 for accepted, 2 for rejected)
            String query = "UPDATE Application SET acceptance = (CASE WHEN gym_id = :gym_id THEN 1 ELSE 2 END) " +
                    "WHERE username = :username";
            session.createQuery(query)
                    .setParameter("gym_id", gym_id)
                    .setParameter("username", username)
                    .executeUpdate();

            // Send email to the trainer
            String email = "default1.wbd@gmail.com";
            String trainer_name = "";
            Application application = ApplicationService.getApplication(session, username, gym_id);
            if (application != null) {
                email = application.getEmail();
                trainer_name = application.getTrainerName();
            }
            
            Email.sendMail(email, true, "GYM " + gym_id, "Trainer " + trainer_name);
            ApplicationService.commitTransaction(session);
            return "Success: Application accepted with username: " + username + " and gym_id: " + gym_id;
        } catch (Exception e) {
            ApplicationService.rollbackTransaction(getSession());
            e.printStackTrace();
            // return "Failed: Application failed with error: " + e.getMessage();
            throw new Exception("Failed: Application failed with error: " + e.getMessage());
        }
    }

    @WebMethod
    public String reject(
        @WebParam(name = "username") String username,
        @WebParam(name = "gym_id") int gym_id
    ) throws Exception {
        try {
            // set the application acceptance to 2
            Session session = ApplicationService.getSession();
            ApplicationService.beginTransaction(session);

            // Update the acceptance status of the application based on the gym_id (1 for accepted, 2 for rejected)
            String query = "UPDATE Application SET acceptance = 2 " +
                    "WHERE username = :username AND gym_id = :gym_id";
            session.createQuery(query)
                    .setParameter("username", username)
                    .setParameter("gym_id", gym_id)
                    .executeUpdate();

            // Send email to the trainer
            String email = "";
            String trainer_name = "";
            Application application = ApplicationService.getApplication(session, username, gym_id);
            if (application != null) {
                email = application.getEmail();
                trainer_name = application.getTrainerName();
            }
            
            Email.sendMail(email, false, "GYM " + gym_id, "Trainer " + trainer_name);
            ApplicationService.commitTransaction(session);
            return "Success: Application rejected with username: " + username + " and gym_id: " + gym_id;
        } catch (Exception e) {
            ApplicationService.rollbackTransaction(getSession());
            e.printStackTrace();
            // return "Failed: Application failed with error: " + e.getMessage();
            throw new Exception("Failed: Application failed with error: " + e.getMessage());
        }
    }

    @WebMethod
    public String resign(
        @WebParam(name = "username") String username
    ){
        try {
            Session session = ApplicationService.getSession();
            ApplicationService.beginTransaction(session);

            // Delete the application based on the username
            String query = "DELETE FROM Application WHERE username = :username";
            session.createQuery(query)
                    .setParameter("username", username)
                    .executeUpdate();
            
            ApplicationService.commitTransaction(session);
            return "Success: Trainer resigned with username: " + username;
        } catch (Exception e) {
            ApplicationService.rollbackTransaction(getSession());
            e.printStackTrace();
            // return "Failed: Resign failed with error: " + e.getMessage();
            return "Failed: Resign failed with error: " + e.getMessage();
        }   
    }

    @WebMethod
    public String getAllStatus(
        @WebParam(name = "username") String username
    ) throws Exception {
        try {
            Session session = ApplicationService.getSession();
            ApplicationService.beginTransaction(session);

            // Get all the gym_id and acceptance status of the aplication
            String query = "SELECT g.gym_id, g.gym_name, g.acceptance, g.application_description FROM Application g WHERE g.username = :username";
            List<Object[]> resultList = session.createQuery(query, Object[].class)
                    .setParameter("username", username)
                    .getResultList();

            ApplicationService.commitTransaction(session);

            StringBuilder result = new StringBuilder();
            for (Object[] status : resultList) {
                int gymId = (int) status[0];
                String gymName = (String) status[1];
                int acceptance = (int) status[2];
                String applicationDescription = (String) status[3];
                result.append("(gym_id: ").append(gymId).append(", gym_name: ").append(gymName).append(", acceptance: ").append(acceptance).append(", application_description: ").append(applicationDescription).append(")\n");
            }

            return result.toString();
        } catch (Exception e) {
            ApplicationService.rollbackTransaction(getSession());
            e.printStackTrace();
            // return "Failed: Failed to retrieve status with error: " + e.getMessage();
            throw new Exception("Failed: Failed to retrieve status with error: " + e.getMessage());
        }
    }

    @WebMethod
    public String getAllAplication(
        @WebParam(name = "gym_id") int gym_id
    ) throws Exception {
        try {
            Session session = ApplicationService.getSession();
            ApplicationService.beginTransaction(session);

            // Get all the application based on the gym_id
            String query = "SELECT a FROM Application a WHERE a.gym_id = :gym_id";
            List<Application> resultList = session.createQuery(query, Application.class)
                    .setParameter("gym_id", gym_id)
                    .getResultList();

            ApplicationService.commitTransaction(session);

            StringBuilder result = new StringBuilder();
            for (Application application : resultList) {
                // result.append(application.toXML()).append("\n");
                result.append(application.toString()).append("\n");
            }

            return result.toString();
        } catch (Exception e) {
            ApplicationService.rollbackTransaction(getSession());
            e.printStackTrace();
            // return "Failed: Failed to retrieve all application with error: " + e.getMessage();
            throw new Exception("Failed: Failed to retrieve all application with error: " + e.getMessage());
        }
    }

    @WebMethod
    public String getAplication(
        @WebParam(name = "username") String username,
        @WebParam(name = "gym_id") int gym_id
    ) throws Exception {
        try {
            Session session = ApplicationService.getSession();
            ApplicationService.beginTransaction(session);

            // Get the application based on the username and gym_id
            String query = "SELECT a FROM Application a WHERE a.username = :username AND a.gym_id = :gym_id";
            Application application = session.createQuery(query, Application.class)
                    .setParameter("username", username)
                    .setParameter("gym_id", gym_id)
                    .uniqueResult();

            ApplicationService.commitTransaction(session);

            // StringWriter stringWriter = new StringWriter();
            // JAXB.marshal(application, stringWriter);
            // return stringWriter.toString();
            // return application.toXML();
            return application.toString();
        } catch (Exception e) {
            ApplicationService.rollbackTransaction(getSession());
            e.printStackTrace();
            // return "Failed: Failed to retrieve application with error: " + e.getMessage();
            throw new Exception("Failed: Failed to retrieve application with error: " + e.getMessage());
        }
    }

}