package com.gymtracker.service;

// import java.io.IOException;
// import java.nio.charset.StandardCharsets;
// import java.nio.file.Files;
// import java.nio.file.Path;
// import java.nio.file.Paths;
import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;

import io.github.cdimascio.dotenv.Dotenv;

public class Email {

    private static final Dotenv dotenv = Dotenv.configure().directory("/").load();
    private static final String username_gt = dotenv.get("USERNAME_GYM_TRACKER");
    private static final String password_gt = dotenv.get("PASSWORD_GYM_TRACKER");
    private static final String host = dotenv.get("SMTP_HOST");
    private static final String port = dotenv.get("SMTP_PORT");
    private static final String website = dotenv.get("WEBSITE_URL");

    public static void sendMail(String recipientEmail, Boolean isAccepted, String gymName, String name) {
        System.out.println("Sending email...");
        System.out.println("username_gt: " + Email.username_gt);
        Properties props = new Properties();
        props.put("mail.smtp.auth", "true");
        props.put("mail.smtp.starttls.enable", "true");
        props.put("mail.smtp.host", Email.host);
        props.put("mail.smtp.port", Email.port);
        props.put("mail.smtp.ssl.protocols", "TLSv1.2");

        Session session = Session.getInstance(props, new Authenticator() {
            protected PasswordAuthentication getPasswordAuthentication() {
                return new PasswordAuthentication(Email.username_gt, Email.password_gt);
            }
        });

        try {
            Message message = new MimeMessage(session);
            message.setFrom(new InternetAddress(Email.username_gt));
            message.setRecipients(Message.RecipientType.TO, InternetAddress.parse(recipientEmail));
            message.setSubject("Gym Tracker Application Status");

            // Read the HTML content from files
            String htmlContent = Email.getHtml(isAccepted);
            // if (isAccepted) {
            //     htmlContent = readFile("acceptance.html");
            // } else {
            //     htmlContent = readFile("rejection.html");
            // }

            // Replace inside the HTML content with the correct values
            htmlContent = htmlContent.replace("{gymName}", gymName);
            htmlContent = htmlContent.replace("{name}", name);
            htmlContent = htmlContent.replace("{website}", Email.website);

            // set the actual HTML content
            message.setContent(htmlContent, "text/html");

            Transport.send(message);
            // System.out.println(htmlContent);

            System.out.println("Email sent successfully.");
        } catch (MessagingException e) {
            e.printStackTrace();
            System.out.println("Failed to send email.");
        }
    }

    // private static String readFile(String filename) {
    //     String content = "";

    //     try {
    //         Path path = Paths.get("src/main/java/com/gymtracker/email/style.html");
    //         byte[] bytes = Files.readAllBytes(path);
    //         String s = new String(bytes, StandardCharsets.UTF_8);
    //         path = Paths.get("src/main/java/com/gymtracker/email/" + filename);
    //         bytes = Files.readAllBytes(path);
    //         String c = new String(bytes, StandardCharsets.UTF_8);
    //         content = s + c;
    //     } catch (IOException e) {
    //         e.printStackTrace();
    //     } 
    //     return content;
    // }

    private static String getHtml(Boolean isAccepted){
        if (isAccepted){
            String result = "<html>\r\n" + //
                    "<head>\r\n" + //
                    "    <style>\r\n" + //
                    "        body {\r\n" + //
                    "            background-color: #f2f2f2;\r\n" + //
                    "            font-family: Arial, sans-serif;\r\n" + //
                    "            margin: 0;\r\n" + //
                    "            padding: 0;\r\n" + //
                    "        }\r\n" + //
                    "        .container {\r\n" + //
                    "            max-width: 600px;\r\n" + //
                    "            margin: 0 auto;\r\n" + //
                    "            padding: 25px;\r\n" + //
                    "            text-align: center;\r\n" + //
                    "        }\r\n" + //
                    "        .content {\r\n" + //
                    "            background-color: #ffffff;\r\n" + //
                    "            border: 1px solid #ccc;\r\n" + //
                    "            padding: 20px;\r\n" + //
                    "            margin-top: 20px;\r\n" + //
                    "        }\r\n" + //
                    "        h1 {\r\n" + //
                    "            color: #333333;\r\n" + //
                    "            margin: 0;\r\n" + //
                    "        }\r\n" + //
                    "        h2 {\r\n" + //
                    "            color: #0073ee;\r\n" + //
                    "            margin-top: 0;\r\n" + //
                    "        }\r\n" + //
                    "        p {\r\n" + //
                    "            color: #555555;\r\n" + //
                    "            margin-top: 10px;\r\n" + //
                    "            margin-bottom: 10px;\r\n" + //
                    "        }\r\n" + //
                    "        a {\r\n" + //
                    "            color: #006bde;\r\n" + //
                    "            text-decoration: none;\r\n" + //
                    "        }\r\n" + //
                    "        a:hover {\r\n" + //
                    "            text-decoration: underline;\r\n" + //
                    "        }\r\n" + //
                    "    </style>\r\n" + //
                    "</head>\r\n" + //
                    "<body>\r\n" + //
                    "    <div class=\"container\">\r\n" + //
                    "        <h1>Gym Tracker</h1>\r\n" + //
                    "        <div class=\"content\">\r\n" + //
                    "            <h2>Congratulations, {name}!</h2>\r\n" + //
                    "            <p>You have been accepted for the application in <strong>{gymName}</strong>.</p>\r\n" + //
                    "            <p>Click <a href=\"{website}\">here</a> to visit our website.</p>\r\n" + //
                    "        </div>\r\n" + //
                    "    </div>\r\n" + //
                    "</body>\r\n" + //
                    "</html>";
            return result;
        } else {
            String result = "<html>\r\n" + //
                "<head>\r\n" + //
                "    <style>\r\n" + //
                "        body {\r\n" + //
                "            background-color: #f2f2f2;\r\n" + //
                "            font-family: Arial, sans-serif;\r\n" + //
                "            margin: 0;\r\n" + //
                "            padding: 0;\r\n" + //
                "        }\r\n" + //
                "        .container {\r\n" + //
                "            max-width: 600px;\r\n" + //
                "            margin: 0 auto;\r\n" + //
                "            padding: 25px;\r\n" + //
                "            text-align: center;\r\n" + //
                "        }\r\n" + //
                "        .content {\r\n" + //
                "            background-color: #ffffff;\r\n" + //
                "            border: 1px solid #ccc;\r\n" + //
                "            padding: 20px;\r\n" + //
                "            margin-top: 20px;\r\n" + //
                "        }\r\n" + //
                "        h1 {\r\n" + //
                "            color: #333333;\r\n" + //
                "            margin: 0;\r\n" + //
                "        }\r\n" + //
                "        h2 {\r\n" + //
                "            color: #0073ee;\r\n" + //
                "            margin-top: 0;\r\n" + //
                "        }\r\n" + //
                "        p {\r\n" + //
                "            color: #555555;\r\n" + //
                "            margin-top: 10px;\r\n" + //
                "            margin-bottom: 10px;\r\n" + //
                "        }\r\n" + //
                "        a {\r\n" + //
                "            color: #006bde;\r\n" + //
                "            text-decoration: none;\r\n" + //
                "        }\r\n" + //
                "        a:hover {\r\n" + //
                "            text-decoration: underline;\r\n" + //
                "        }\r\n" + //
                "    </style>\r\n" + //
                "</head>\r\n" + //
                "<body>\r\n" + //
                "    <div class=\"container\">\r\n" + //
                "        <h1>Gym Tracker</h1>\r\n" + //
                "        <div class=\"content\">\r\n" + //
                "            <h2>Sorry, {name}</h2>\r\n" + //
                "            <p>We regret to inform you that your application for {gymName} has been rejected</p>\r\n" + //
                "            <p>Click <a href='{website}'>here</a> to visit our website for more information</p>\r\n" + //
                "        </div>\r\n" + //
                "    </div>\r\n" + //
                "</body>\r\n" + //
                "</html>";
            return result;
        }   
    }

    public static void main(String[] args) {
        String recipientEmail = "trainer1@gmail.com";
        String gymName = "Example Gym";
        String name = "Nigel Sahl";
        Boolean isAccepted = true;

        Email.sendMail(recipientEmail, isAccepted, gymName, name);
    }
}